#!/bin/sh

# no network
test "$1" = "-" && { export ALL_PROXY=socks5://localhost; shift; }
# git clone http://pkgs.fedoraproject.org/git/rpms/cups.git/ $FCDISTRO
export UBCNAME="$(curl -s https://packages.ubuntu.com/ | sed -n '/Browse/,/\/ul/{/<li/h};${x;s/.*"\([^"]*\)\/".*/\1/p}')"

# TODO getoptize

RPMH="$HOME/RPM"
NNAME=`find $RPMH/SPECS -name \*.spec | tail -1`
NAME="${1:-$NNAME}"
# if specfile or no $1 is provided
NAME="${NAME##*/}"; NAME="${NAME%.spec}"
TMDISTRO="${TMDISTRO:-$HOME/others}"
FCDISTRO="${FCDISTRO:-$TMDISTRO/fc}"
UBDISTRO="${UBDISTRO:-$TMDISTRO/debian}"
SRCSPEC=""
DSTSRC="$RPMH/SOURCES"
PATCHLOG="$DSTSRC/README.patches"
DSTSPEC="$DSTSRC/$NAME.spec"
DSTBUILD="$RPMH/BUILD/`ls $RPMH/BUILD/ | tail -1`"
FCID="FC"
UBID="Ubuntu"
# This files can set up UNUSED_FC, FAILED_FC, UNUSED_UB and FAILED_UB"
UNUSED_FC=""; FAILED_FC=""; UNUSED_UB=""; FAILED_UB=""
for n in "$0" "`basename $0`" "$DSTSRC/`basename $0`" "$DSTSRC/$NAME"; do
  test -r "$n.unused" && { echo "Using $n.unused"; . "$n.unused"; } || continue
  break
done

mkdir -p "$TMDISTRO"
test -d "$FCDISTRO" || git clone https://src.fedoraproject.org/rpms/$NAME.git "$FCDISTRO"
#test -d "$UBDISTRO" || wget -q -O- `wget -q -O- "http://packages.ubuntu.com/$UBCNAME/$NAME" | grep 'debian.tar.[gx]z' | sed 's@.*href="\([^"]*\).*@\1@'` | tar x -C "$TMDISTRO" -f -
echo "	Using Ubuntu $UBCNAME"
test -d "$UBDISTRO" || {
  UBDIFF=`curl -s "https://packages.ubuntu.com/$UBCNAME/$NAME" | grep 'debian.tar.[gx]z' | sed 's@.*href="\([^"]*\).*@\1@'`
  case "$UBDIFF" in
    *gz) UBXDIFF="x";;
    *xz) UBXDIFF="J";;
  esac
  curl -s "$UBDIFF" | tar x$UBXDIFF -C "$TMDISTRO" -f -
}

mkdir -p "$DSTSRC"
for S in $NAME.spec $RPMH/SPECS/$NAME.spec; do
  test -r $S && { SRCSPEC="$S"; break; }
done

delGENpatches() { # e-regexp
  local F="$1" T
  shift
  for T; do
    sed -i -r '/^## '"$T"' /,/^## /{
    /^## '"$T"' /a
    /^[^#]|^[[:space:]]*$|^#+patch/d
    }' "$F"
  done
}

insPatch() { # patchfile patchargs commented distro patchdir
  local a DID="$4" DPD="$5"
  local pn="$DID-${1#$NAME-}"
  pn="${pn%.diff}"; pn="${pn%.patch}.patch"
  test "$3" == "" && a="%" || a="$3"

  sed -Ei "
  /^## $DID patches/,/^$/{
    /^$/iPatch$NPATCH: $pn
  }" "$DSTSPEC" &&
  sed -Ei "
  /^## $DID apply patches/,/^$/{
    /^$/i${a}patch$NPATCH $2
  }" "$DSTSPEC" &&
  cp "$DPD/$1" "$DSTSRC/$pn" &&
  echo "$a $DID $1" >> "$PATCHLOG"
  NPATCH="$(($NPATCH + 1))"
}

chkDebPatches() { # patchdir dir [patch parameters]
  local P="`realpath "$1"`" D="`realpath "$2"`"
  shift; shift
  cd "$D"; pwd
  git init
  git config user.email "you@example.com"
  git config user.name "Your Name"
  git add .
  git commit -a -q -m "INIT" || :
  git tag -d INIT || : > /dev/null 2>&1
  git tag INIT || return 1
  for p in `grep "^[^#].*" "$P/series"`; do
    echo "### $p" >&2
    if patch "$@" < "$P/$p" >&2 ; then
      git add .
      # > /dev/null
      git commit -a -q -m "$p"
      echo "$p"
    else
      git reset -q --hard
      git clean -qfd
      echo "## $p"
    fi
  done
}

chkPatchGone() { # patches where_to_check [name]
  for p in $1; do
    fgrep -q "$p" "$2" || echo "Warning: $p is not found in ${3:-$2}"
  done
}

cp "$SRCSPEC" "$DSTSPEC"
delGENpatches "$DSTSPEC" "$FCID" "$UBID"
:>"$PATCHLOG"

# FC
rm -f "$DSTSRC/$FCID-"*

# store FC names
#eval `sed -En 's/^(Patch[[:digit:]]+):[[:space:]]*(.+)/\1="\2";/p' "$FCDISTRO/$NAME.spec"`
eval `sed -En '/^Patch[[:digit:]]+/{
s/^(Patch[[:digit:]]+):[[:space:]]*(.+)/\1="\2";/
'"s/%name|%\{name\}/$NAME/g"'
p
}
' "$FCDISTRO/$NAME.spec"`

ChkUnused() { # what unused_list failed_list
  local f
  f="${1#$NAME-}"; f="${f%.diff}"; f="${f%.patch}"; f="${f#0*-}"
  if echo "$2" | fgrep -q "$f"; then
    echo "Unused $f" >&2
    echo -n "#"
  elif echo "$3" | fgrep -q "$f"; then
    echo "Failed $f" >&2
    echo -n "##"
  fi
}

# apply FC patches
NPATCH="1"
# If no %patch found, suppose another patch scheme is used (e. g. git am)
{ egrep '^%patch[[:digit:]]+[[:space:]]' "$FCDISTRO/$NAME.spec" ||
  sed -En 's/^Patch([[:digit:]]+):.*/%patch\1 -p1/p' "$FCDISTRO/$NAME.spec" | sort -nt- -s
} | 
  while read cmd args; do
    n="${cmd#%patch}"
    eval 'f="$Patch'$n'"'
    unused="`ChkUnused "$f" "$UNUSED_FC" "$FAILED_FC"`"
    insPatch "$f" "$args" "$unused" $FCID "$FCDISTRO"
  done
chkPatchGone "$UNUSED_FC" "$FCDISTRO/$NAME.spec" FC
chkPatchGone "$FAILED_FC" "$FCDISTRO/$NAME.spec" FC

# apply UB patches
NPATCH="101"
for f in `grep "^[^#].*" "$UBDISTRO/patches/series"`; do
  unused="`ChkUnused "$f" "$UNUSED_UB" "$FAILED_UB"`"
  insPatch "$f" "-p1" "$unused" "$UBID" "$UBDISTRO/patches"
done
chkPatchGone "$UNUSED_UB" "$UBDISTRO/patches/series" Ubuntu
chkPatchGone "$FAILED_UB" "$UBDISTRO/patches/series" Ubuntu

test -z "$CHK" || chkDebPatches "$UBDISTRO/patches" "$DSTBUILD" -p 1 2>&1 | tee $HOME/.log.tmp
